/*
 * JAEBIServer.java
 *
 * Created on June 14, 2005, 5:30 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.jaebi.server.auth.Session;
import org.jaebi.server.core.TimeoutThreadedUserContainer;
import org.jaebi.server.core.User;
import org.jaebi.server.core.UserContainer;
import org.jaebi.server.exception.JAEBIServerExecException;
import org.jaebi.server.exception.JAEBIServerInitException;
import org.jaebi.server.exception.JAEBIServerUserRegistrationException;
import org.jaebi.server.exception.JAEBIServerWrongSessionException;
import org.jaebi.server.service.ServiceMediator;
import org.jaebi.server.service.ServiceMediatorImpl;
import org.jaebi.server.service.exception.ServiceMediatorExecException;
import org.jaebi.server.service.exception.ServiceMediatorInitException;

/**
 * @class JAEBIServer
 * @author detro
 *
 * Server di JAEBI.
 * Questa Classe e' si colloca al massimo livello di astrazione
 * possibile per questo progetto.
 *
 * Una volta istanziata, alloca tutte le risorse 
 * necessarie a fornire il servizio
 * ed incapsula la logica di interezione tra tutte le classi di JAEBI, 
 * sfruttando anche la classe ServiceMediator, che rende semplice
 * l'utilizzo dei servizi offerti in questa implementazione di JAEBI.
 */
public class JAEBIServer {
	/** Container di User registrati sul Server */
    private final UserContainer registeredUsers;
	/** Istanza di Service Mediator che incapsula e 
	 * centralizza l'uso di tutti i Servizi */
    private ServiceMediator serviceMediator;
    private static final String exceptionPrefix = "[JAEBIServer Exception]: ";
    
    /** 
     * Creates a new instance of JAEBIServer
     * 
     * @param backEndPropertiesFilePath File Path al file di configurazione
     * del BackEndHandler
     * 
     * @exception JAEBIServerInitException Errore in fase di Inizializzazione
     * del JAEBI Server
     */
    public JAEBIServer(String backEndPropertiesFilePath) 
        throws JAEBIServerInitException {
        
        /* Alloca il container di User:
         * controlla ogni 30 secondi che uno User non sia inattivo
         * da piu' di 5 minuti */
		// param 01: Elimina User dopo 5 min (300 sec) di inattivita'
		// param 02: Controlla Timestamp degli User ogni 30 sec
        registeredUsers = new TimeoutThreadedUserContainer( 300000, 30000 ); 
        serviceMediator = null;
        
        try {
            /* Alloca il Service Mediator */
            if ( backEndPropertiesFilePath != null ) {
                serviceMediator = 
                        new ServiceMediatorImpl(backEndPropertiesFilePath);
            } else {
                serviceMediator = new ServiceMediatorImpl();
            }
        } catch ( ServiceMediatorInitException sminite ) {
            throw new JAEBIServerInitException( 
                    exceptionPrefix + "Server Initialization Error\n" + sminite );
        }
    }
	
	/** 
     * Creates a new instance of JAEBIServer
     * 
     * @exception JAEBIServerInitException Errore in fase di Inizializzazione
     * del JAEBI Server
     */
	public JAEBIServer() 
            throws JAEBIServerInitException {

            this(null);
	}
    
    /**
     * Registra un nuovo User sul Server.
     *
     * @param userNick NickName dello User
     * @param userSession Session assegnata allo User 
     * (che lo identifica univocamente all'interno del Server)
     *
     * @param userType Una stringa contenente il tipo di Utente
     * @exception JAEBIServerUserRegistrationException In caso ci siano
     * dei problemi durante la registrazione del nuovo User
     */
    public void registerNewUser(String userNick, 
            Session userSession, String userType) 
        throws JAEBIServerUserRegistrationException {
        
        if ( !registeredUsers.addUser(userNick, userSession, userType) ) {
            throw new JAEBIServerUserRegistrationException(
                    exceptionPrefix + "Unable to Register User\n" );
        }
    }
    
    /**
     * Elimina uno User registrato sul Server.
     *
     * @param userSession Session che identifica univocamente uno User
     * @exception JAEBIServerWrongSessionException In caso di problemi
     * con il Session fornito (per esempio, non esistono User con quel
     * Session)
     */
    public void unregisterUser(Session userSession) 
        throws JAEBIServerWrongSessionException {
        
        if ( !registeredUsers.removeUser(userSession) ) {
            throw new JAEBIServerWrongSessionException(
                    exceptionPrefix + "User or Session Unknown\n" );
        }
    }
    
    /**
     * Recupera uno User Registrato dal Server
     *
     * @param userSession Session che identifica univocamente uno User
     * @return User trovato
     * @exception JAEBIServerWrongSessionException In caso di problemi
     * con il Session fornito (per esempio, non esistono User con quel
     * Session)
     */
    public User getRegisteredUser(Session userSession)
        throws JAEBIServerWrongSessionException {
        
        if ( !registeredUsers.contains(userSession) ) {
            throw new JAEBIServerWrongSessionException(
                    exceptionPrefix + "User or Session Unknown\n" );
        } else {
            return registeredUsers.getUser(userSession);
        }
    }
    
    /**
     * Esegue una richiesta
     *
     * La richieta dovra' essere codificata, in questa implementazione,
     * nell'XML Xrequest. La risposta sara' codificata nell'XML
     * Xresponse.
     * 
     * @param userSession Identifica quale User ha eseguito la richiesta
     * @param userRequest Richiesta XML
     * @exception JAEBIServerExecException Errore durante l'esecuzione
     * della Requst.
     */
    public String executeRequest(Session userSession, String userRequest) 
        throws JAEBIServerExecException, JAEBIServerWrongSessionException {
        
        User requestingUser = null;
        String result = null;
        
        requestingUser = getRegisteredUser(userSession);
                
        try {
            result = serviceMediator.executeServiceToString(requestingUser, userRequest);
        } catch (ServiceMediatorExecException sermedee) {
            throw new JAEBIServerExecException(
                    exceptionPrefix + "Service Error\n" + sermedee );
        }
        
        return result;
    }
    
	/**
	 * Ritorna una List di tutti gli User attualmente Registrati
	 * 
	 * @return List degli User registrati
	 */
	public List getRegisteredUsers() {
		Collection coll = registeredUsers.toCollection();
		ArrayList result = new ArrayList();
				
		result.addAll(coll);
		
		return result;
	}
}
