package org.jaebi.server;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import org.jaebi.server.auth.Authenticator;
import org.jaebi.server.auth.AuthenticatorImpl;
import org.jaebi.server.auth.Session;
import org.jaebi.server.core.User;
import org.jaebi.server.exception.JAEBIServerExecException;
import org.jaebi.server.exception.JAEBIServerInitException;
import org.jaebi.server.exception.JAEBIServerUserRegistrationException;
import org.jaebi.server.exception.JAEBIServerWrongSessionException;


/**
 * JAEBI Web Service
 * @class JAEBIWSImpl
 * @author detro
 * @version 1.0alpha
 * 
 * Questa Classe e' uno "skeleton" per il WS che andremo a generare.
 * <p>
 * I metodi qui' definiti saranno mappati dal tool 
 * <i><b>WSCompile</b></i> 
 * (<a href="http://java.sun.com/webservices/docs/1.1/tutorial/doc/JAXRPC6.html">link</a>) 
 * in un Web Service di tipo <b><u>Document/Literal</u></b>.<br>
 * Il Web Service disporra' di un <b>WSDL 1.x (Web Service Definition Language)</b>, 
 * un file XML (Standard W3c) scaricabile dal WS stesso, 
 * che definisce la "forma" del Web Service
 * e permette la creazione di Client WS senza dover conoscere ASSOLUTAMENTE
 * la reale implementazione dello stesso.<br>
 * Esistono linguaggi come PHP che forniscono Framework per creare Client WS
 * letteralmente "on-the-fly": cio' e' realizzabile grazie al WSDL. 
 * </p>
 * 
 * <p>
 * Inizialmente si era pensato di implementare il Web Service secondo lo
 * "stile" XML-RPC perche', al momento, 
 * e' l'unico che offre la sicurezza di funzionamento senza dover 
 * scrivere classi per la Serializzazione/Deserializzazione. <br>
 * Infatti, per il funzionamento interno di JAEBI, 
 * sarebbe stato piu' conveniente poter
 * spedire tramite WebService interi oggetti.
 * Esigenze di sviluppo pero' ci hanno costretti a sperimentare
 * uno "stile" piu' nuovo di Web Service detto 
 * <b><u>Document-Type Web Service</u></b>;
 * cio' e' stato causato dal fatto che i client J2ME, al momento, dispongono
 * solo del Toolkit WTK2.2 di Sun, il quale supporta SOLO i Web Service
 * "Document-Styled".<br>
 * Questo standard definisce in maniera troppo generica come eseguire
 * il mapping "Oggetto di un particolare Linguaggio"-"XSD", facendo si
 * che WebService scritti con Framework come <b><i>.NET</i></b>, 
 * non siano affatto compatibili con
 * quelli scritti con AXIS (<b>Apache</b>) o 
 * <b><i>WSCompile</i></b> (<b>Sun</b>).
 * </p>
 *  
 * <p>
 * Questa limitazione e' pero' circoscritta al solo WS (quindi, solo questa Classe):
 * quando (e se) sara' disponibile il nuovo standard WSDL 2.0 che definisca
 * con maggiore precisione i WS, il passaggio al tipo "Document" sarebbe
 * veloce ed indolore per tutta l'archiettura di JAEBI.
 * </p>
 */
public class JAEBIWSImpl implements JAEBIWSSEI {
    /** Server JAEBI istanziato dal Web Service */
    private JAEBIServer coreServer;
    /** Authenticator istanziato dal Web Service */
    private final Authenticator authenticator;
    /** HashMap che server a recuperare le Session degli User.
     * Inizialmente avevamo pensato di inviare direttamente l'oggetto
     * Session ai Client i quali dovevano poi reinviarlo ad ogni
     * iterazione: questa cosa pero' non e' risultata realizzabile, 
     * costringendoci ad ideare un sistema di Mapping che invia ai Client
     * l'hashCode degli oggetti Session (definito SessionID), costruendo intanto
     * una HashMap server-side che utilizzi questi valori come chiave e
     * le corrispettive Session come valori: ci permette il recupero 
     * dell'associazione tra User e Session. */
    private final HashMap activeSessions;

    /** Configuration BaseDir */
    private static final String configBaseDir = 
            "/opt/tomcat/webapps/axis/JAEBIWS/config/";
    /** BackEndHandler Property File Name */
    private static final String backEndPropFile = 
            "BackEndHandlerImpl.properties";

    /** WrongSessionErrorID */
    private static final String messageWrongSessionErrorID = 
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
            "<xresponse xmlns='http://xml.netbeans.org/examples/targetNS' " +
            "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' " +
            "xsi:schemaLocation='http://xml.netbeans.org/examples/targetNS " +
            "xresponse.xsd'>" +
            "<message result=\"error\">" +
            "Received a Wrong Session ID: please, check it." +
            "</message>" +
            "</xresponse>";

    /** ServiceExecError */
    private static final String messageServiceError = 
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
            "<xresponse xmlns='http://xml.netbeans.org/examples/targetNS' " +
            "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' " +
            "xsi:schemaLocation='http://xml.netbeans.org/examples/targetNS " +
            "xresponse.xsd'>" +
            "<message result=\"error\">" +
            "Error during Service Execution." +
            "</message>" +
            "</xresponse>";

    /** SessionError */
    private static final String messageSessionError = 
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
            "<xresponse xmlns='http://xml.netbeans.org/examples/targetNS' " +
            "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' " +
            "xsi:schemaLocation='http://xml.netbeans.org/examples/targetNS " +
            "xresponse.xsd'>" +
            "<message result=\"error\">" +
            "User Session Error." +
            "</message>" +
            "</xresponse>";

    /** Logout */
    private static final String messageLogout = 
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
            "<xresponse xmlns='http://xml.netbeans.org/examples/targetNS' " +
            "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' " +
            "xsi:schemaLocation='http://xml.netbeans.org/examples/targetNS " +
            "xresponse.xsd'>" +
            "<message result=\"error\">" +
            "Logged Out. Bye from JAEBI Server." +
            "</message>" +
            "</xresponse>";

    /** Last Error Message */
    private String lastErrorMessage = null;
	
    /**
     * Costruttore del JAEBI Web Service.
     * 
     * Il WS di JAEBI espone i metodi necessari e sufficienti per i client
     * per interagire con il Server.
     * 
     * Il WS si occupa di gestire:
     * <ul>
     *  <li>
     * 	Una istanza di JAEBIServer, 
     * 	che incapsula tutta la logica di funzionamento del Server
     *  </li>
     *  <li> 
     *      Una istanza di Authenticator, che
     *	assegna Session (Identificativi) agli User che
     *	effetuano il Login
     *  </li>
     * </ul>
     * 
     * Il costruttore legge da File i Messaggi predefiniti e
     * li carica in opportuni StringBuffer: si sarebbero potute utilizzare
     * le Classi del package "org.jaebi.server.service.xml" per costruire e
     * serializzare (marshallare) gli "xresponse", ma cio' avrebbe comportato
     * un sovraccarico prestazionale inutile, dato che questi messaggi possono
     * cambiare (e cio' comporterebbe la necessita' di correggere direttamente
     * il sorgente).
     */
    public JAEBIWSImpl() {
        // Inizializzazione Authenticator
        authenticator = new AuthenticatorImpl();

        // Inizializzazione "archivio Sessioni"
        activeSessions = new HashMap();

        // Inizializzazione JAEBIServer
        try {
            coreServer = new JAEBIServer( backEndPropFile );
        } catch (JAEBIServerInitException e) {
            /* Nel caso si verifichi qui un errore,
             * sarebbe irrimediabile per il corretto
             * funzionamento del Server: conserviamo in ogni 
             * caso il messaggio circa l'errore prodotto.
             */
            lastErrorMessage = e.getMessage();
        }
    }
    
    /**
     * Metodo di Login sul JAEBI Server.
     * 
     * @param userNick Nick identificativo dell'User (a piacere)
     * @param userType Tipo di User. Per ora, non esistono Tipi predefiniti di
     * User, quindi si e' preferito non fissare dei limiti.
     * Nelle classi Client si aggiungera' una Interfaccia con le costanti
     * che si consiglia di utilizzare.
     * 
     * @return SessionID: hashCode della Sessione assegnata allo User. 
     * I client dovranno inviare SessionID ad ogni loro interazione con
     * JAEBI. Torna "-1" in caso di errore.
     */
    public int login(java.lang.String usernick, java.lang.String usertype) {
        Session userSession;
		
        // Allochiamo un Session per il nuovo User
        userSession = authenticator.getAuthentication(usernick);

        /* Registriamo il Session, indicizzato in base al SessionID (in pratica, 
         * l'hashcode di Session) */ 
        activeSessions.put(
            new Integer( userSession.hashCode() ), 
            userSession);

        if ( coreServer == null )
            return -1;

        // Registrazione nuovo User
        try {
            coreServer.registerNewUser(usernick, userSession, usertype);
        } catch (JAEBIServerUserRegistrationException e) {
            lastErrorMessage = e.getMessage();
            return -1;
        }

        /* TODO
         * Cercare un algoritmo per la generazione del SessionID
         * piu' sicuro: l'HASHCODE potrebbe generare un valore
         * uguale al valore di errore (-1), creando confusione
         */
        return userSession.hashCode();
    }
    
    /**
     * Metodo di Logout sul JAEBI Server.
     * 
     * @param userSessionID SessionID / Hashcode che i Client devono
     * inviare ad ogni interazione con il Server.
     */
    public String logout(int sessionid) {
        Session userSession;
        String xresponse = null;

        // Recupero oggetto Session in base al SessionID 
        userSession = (Session)activeSessions.get( new Integer(sessionid) );

        if ( userSession == null ) {
            xresponse = messageWrongSessionErrorID;
        } else {
            // Deregistrazione User
            try {
                coreServer.unregisterUser(userSession);
                xresponse = messageLogout;
            } catch (JAEBIServerWrongSessionException e) {
                xresponse = messageSessionError;
                lastErrorMessage = e.getMessage();
            }
        }

        return xresponse;
    }
    
    
    /**
     * Ritorna il Messaggio relativo all'ultimo Errore verificatori.
     * 
     * @return Ultimo Errore verificatosi nel Server
     */
    public String getLastError() {
        return lastErrorMessage;
    }

    /**
     * Esegui Request XML sul Server (Xrequest).
     * 
     * @param sessionid Identificativo Sessione dell'User
     * @param xrequest XML che mappa la richiesta
     * @return Stringa in formato XML Xresponse che mappa 
     * il risultato della richiesta
     */
    public String executeRequest(int sessionid, java.lang.String xrequest) {
        String xresponse = null;
        Session userSession = (Session)activeSessions.get( new Integer(sessionid) );

        if ( userSession == null ) {
            // Il "sessionId" fornito e' errato
            xresponse = messageWrongSessionErrorID;
        } else {
            // Esecuzione Richiesta
            try {
                xresponse = coreServer.executeRequest( userSession, xrequest);
            } catch (JAEBIServerExecException e) {
                lastErrorMessage = e.getMessage();
                xresponse = messageServiceError;
            } catch (JAEBIServerWrongSessionException e) {
                lastErrorMessage = e.getMessage();
                xresponse = messageSessionError;
            }
        }

        return xresponse;
    }

    /**
     * Ritorna una Stringa contenente la Lista degli User attualmente
     * registrati sul Server.
     * 
     * @return Lista di User registrati
     */
    public String getUsersList() {
        StringBuffer result = new StringBuffer();
        Collection coll = activeSessions.values();
        Iterator iter = coll.iterator();
        Session currentSession = null;
        User currentUser = null;
        int i = 0;

        result.append("[JAEBIWS] REGISTERED USERS:\n");
        while ( iter.hasNext() ) {
            currentSession = (Session)iter.next();

            try {
                currentUser = coreServer.getRegisteredUser(currentSession);
                result.append("[" + ++i + "] - " + currentUser.getNick() + "\n" );
            } catch (JAEBIServerWrongSessionException e) {
                lastErrorMessage = e.getMessage() 
                    + "Session ID: " + currentSession.toString();
                //iter.remove();
            }
        }

        return result.toString();
    }

    /**
     * Ritorna il numero di User attualmente registrati sul Server.
     * 
     * @return Numero di User registrati
     */
    public int getUsersNumber() {
        StringBuffer result = new StringBuffer();
        Collection coll = activeSessions.values();
        return coll.size();
    }
}
