/**
 * 
 */
package org.jaebi.server.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;

import org.jaebi.server.auth.Session;

/**
 * @author detro
 * @class TimeoutThreadedUserContainer
 * 
 * <p>
 * Questa specializzazione della classe UserContainerImpl estende le
 * funzionalita' del Container originale, associando ad ogni Session un
 * <u>Timestamp</u>: il Timestamp indica l'ultimo istante in cui lo User
 * ha eseguito una qualche operazione sul Server.
 * <br>
 * Quando il Timestamp scade (cioe' quando viene superato un determinato Timeout, 
 * selezionato all'istanziazione del Container), 
 * lo User e la relativa Session vengono
 * rimossi dal Server forzatamente.
 * <br>
 * Tutto cio' e' stato realizzato perche' l'interfacciamente a JAEBI, malgrado
 * esser basato su autenticazione, non e' "<i>connection-oriented</i>": basandosi su
 * "<i><b>Web Services-over-HTTP</b></i>" 
 * (l'unica tipologia di WS attualemente utilizzata),
 * non esiste una Connessione fisica basata su <u>Socket</u> (come, ad esempio, in un
 * Server FTP).
 * <br>
 * E' quindi necessaria una sorta di "<i>Garbage-Collecting</i>" degli User non 
 * piu' attivi ma che non hanno richiesto esplicitamente il logout.
 * </p>
 * <p>
 * La Classe implementa Runnable e, nel costruttore, fa partire un Thread
 * con un ciclo infinito che controlla continuamente (ma con una pausa fissata per
 * evitare inutile sovraccarico del Sistema) che non ci siano User che 
 * hanno superato il Timeout.
 * </p>
 */
public class TimeoutThreadedUserContainer 
	extends UserContainerImpl implements Runnable {
	
	/**
	 * Classe Container utile ad associare un Timestamp
	 * di "ultimo accesso" ad una Session di uno User.
	 * 
	 * Questa Classe permette di capire quando una Session risulta
	 * (in gergo WEB-Developer) "scaduta".
	 * 
	 * @author detro
	 */
	class SessionLastAccessTimestamp {
		private final Session session;
		private long lastAccessTimestamp;
		
		/** Costruttore Pair 
		 * 
		 * @param newSession Session a cui associare il Timestamp
		 */
		SessionLastAccessTimestamp ( Session newSession ) {
			session = newSession;
			update();
		}
		
		/** Aggiorna il Timestamp */
		public synchronized void update() {
			lastAccessTimestamp = new GregorianCalendar().getTimeInMillis();
		}

		/** 
		 * @return Returns the session.
		 */
		public Session getSession() {	return session; }
		
		/**
		 * @return Timestamp ultimo Accesso
		 */
		public synchronized long getLastAccessTimestamp() { return lastAccessTimestamp; }
	}
	
	/** HashMap che contiene oggetti di tipo SessionLastAccessTimestamp.
	 * Questi oggetti saranno indicizzati in base al nome degli User per
	 * velocizzare il loro recupero.
	 */
	private final HashMap timedSessions;
	
	/** Intervallo tra un controllo sul Timeout User 
	 * e il successivo */
	private final long timeOutCheckIntervall;
	/** Tempo di Timeout per gli User */
	private final long userTimeOut;
	/** Tempo di Sleep del Thread di "Garbage-Collecting" */
	private static final long sleepTime = 10000; // 10 sec
	
	/** Costruttore TimeoutThreadedUserContainer
	 * 
	 * @param checkTimeIntervall Intervallo di tempo che scandisce quando
	 * controllare se gli User hanno superato il tempo Max di TimeOut per
	 * essere automaticamente eliminati. In <b>millisecondi</b>.
	 * 
	 * @param userTimeOut Tempo di TimeOut. In <b>millisecondi</b>.
	 */
	public TimeoutThreadedUserContainer(long userTimeOut, long timeOutCheckIntervall) {
		super();
		
		timedSessions = new HashMap();
		this.userTimeOut = userTimeOut;
		this.timeOutCheckIntervall = timeOutCheckIntervall;
		new Thread(this).start();
	}
	
	/**
	 * Aggiunge uno User.
	 * 
	 * Gestisce anche la creazione del relativo oggetto
	 * SessionLastAccessTimestamp.
	 * 
	 * @see UserContainer
	 */
	public synchronized boolean addUser( User newUser ) {
		timedSessions.put(
				newUser.getNick(), 
				new SessionLastAccessTimestamp( newUser.getSession() )
				);
        return super.addUser( newUser );
    }
	
	/**
	 * Aggiunge uno User.
	 * 
	 * Gestisce anche la creazione del relativo oggetto
	 * SessionLastAccessTimestamp.
	 * 
	 * @see UserContainer
	 */
	public synchronized boolean addUser( String nick, Session session, String type ) {
		timedSessions.put(
				nick, 
				new SessionLastAccessTimestamp( session )
				);

		return super.addUser( nick, session, type);
	}
	
	/**
	 * Gestisce anche la rimozione del relativo oggetto
	 * SessionLastAccessTimestamp.
	 *  
	 * @see UserContainer */
	public synchronized boolean removeUser( Session session ) {
		User user = super.getUser(session);
		if ( user != null )
			timedSessions.remove( user.getNick() );
		
		return super.removeUser(session);
    }
	
	/**
	 * Gestisce anche l'aggiornamento del relativo oggetto
	 * SessionLastAccessTimestamp.
	 * 
	 *  @see UserContainer */
	public synchronized User getUser( Session session ) {
		User user = super.getUser(session);
		if ( user != null ) {
			SessionLastAccessTimestamp sessionLastAccess = 
				(SessionLastAccessTimestamp)timedSessions.get( user.getNick() );
			
			sessionLastAccess.update();
		}
		
		return user;
    }
	
	/**
	 * Flusso di Controllo per l'eliminazione
	 * degli User che non si connettono da troppo tempo.
	 * 
	 * Controlla i Timestamp relativi all'ultimo accesso degli
	 * User e, se troppo inveriore rispetto al Timestamp corrente,
	 * lo User viene eliminato.
	 */
	public void run() {
		// Recupero Set contenente tutte le SessionLastAccessTimestamp registrate
		Collection timedSessionsColl = timedSessions.values();
		Iterator iter;
		SessionLastAccessTimestamp currSessionLastAccess;
		long actualTimestamp;
		// Collezione delle Sessioni degli User da eliminare
		ArrayList toDelete = new ArrayList();
		
		// Ciclo Indefinito: continua sino alla distruzione dell'Istanza
		while ( true ) {
			// Iteratore sugli oggetti SessionLastAccessTimestamp
			iter = timedSessionsColl.iterator();
			// Timestamp attuale in millisecondi
			actualTimestamp = new GregorianCalendar().getTimeInMillis();
			// Iteriamo sui SessionLastAccessTimestamp
			while ( iter.hasNext() ) {
				currSessionLastAccess = (SessionLastAccessTimestamp)iter.next();
				// Se lo User e' inattivo da troppo tempo
				if ( 
					( actualTimestamp - 
					currSessionLastAccess.getLastAccessTimestamp() )
					>= this.userTimeOut
				) {
					// Conservazione della Session da Eliminare
					toDelete.add( currSessionLastAccess.getSession() );
				}
			}
			
			// Iteratore sugli oggetti Session da Eliminare
			iter = toDelete.iterator();
			while ( iter.hasNext() ) {
				// Rimozione User relativo alla Session "scaduta"
				removeUser( (Session)iter.next() );
			}
			// Pulizia della lista di Session da Eliminare
			toDelete.clear();
			// Suggerimento per la JVM di pulire la memoria
			System.gc();
			
			try {
				Thread.sleep(TimeoutThreadedUserContainer.sleepTime);
			} catch (InterruptedException e) { }
		}
	}

}
