/*
 * BackEndHandlerImpl.java
 *
 * Created on June 5, 2005, 8:12 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server.service;

import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import org.jaebi.server.service.exception.BackEndHandlerGenericException;
import org.jaebi.server.service.exception.BackEndHandlerInitException;
import org.jaebi.server.service.exception.BackEndHandlerSQLCreationException;

/**
 * @class BackEndHandlerImpl
 * @author detro
 *
 * Questa Classe e' un esempio di implementazione dell'interfaccia
 * BackEndHandler.
 *
 * Questa implementazione realizza un interfaccia verso un
 * DataBase. In questo esempio, l'interfaccia espone pochi e semplici
 * metodi.
 */
public class BackEndHandlerImpl implements BackEndHandler {
    /** DSN - Data Source Name: Descrive la connessione al DB */
    private final String DSN;
    /** Username dell'utente che si connette al DB */
    private final String username;
    /** Password */
    private final String password;
    /** 
     * Nome del Driver JDBC da utilizzare: caricare a RunTime il Driver
     * JDBC permette di cambiare RDBMS e Driver senza modificare (nei limiti
     * del possibile) l'implementazione */
    private final String JDBCDriverName;
    /** Default File delle Proprieta' del BackEnd (configurazione) */
    private static String defaultPropertyFilePath = 
            "/Users/detro/workspace-nb/L2-Project-JAEBI/JAEBI/BackEndHandlerImpl.properties";
    /** File delle Proprieta' del BackEnd caricato */
    private final String loadedPropertyFilePath;
    private static final String exceptionPrefix = "[BackEndHandlerImpl Exception]: ";
    /** Classe del Driver JDBC */
    private final Class driverClass;
    /** Handler Connessione al DB */
    private final Connection DBConnection;
        
    /**
     * Creates a new instance of BackEndHandlerImpl.
     * 
     * Use the Default File Path to the Property File.
     * @exception BackEndHandlerInitException Problemi durante 
     * l'inizializzazione dell'Oggetto (Property File mancante, 
     * Driver JDBC mancante, ecc.)
     */
    public BackEndHandlerImpl() throws BackEndHandlerInitException {
        this( defaultPropertyFilePath );
    }
    
    /**
     * Creates a new instance of BackEndHandlerImpl.
     * 
     * @param propertyFilePath Specify the Property File Path to be used
     * @exception BackEndHandlerInitException Problemi durante 
     * l'inizializzazione dell'Oggetto (Property File mancante, 
     * Driver JDBC mancante, ecc.)
     */
    public BackEndHandlerImpl(String propertyFilePath) throws BackEndHandlerInitException {
        // Load the Property File
        Properties config = new Properties();

        try {
            FileInputStream configStream = new FileInputStream(propertyFilePath);
            config.load(configStream);
            configStream.close();
        } catch (IOException ec) {
            throw new BackEndHandlerInitException( exceptionPrefix + ec );
        }
        
        DSN = config.getProperty("DSN");
        username = config.getProperty("username");
        password = config.getProperty("password");
        JDBCDriverName = config.getProperty("JDBCDriverName");
        loadedPropertyFilePath = propertyFilePath;

        // Driver Registration
        try {
            driverClass = Class.forName(JDBCDriverName);
            DriverManager.registerDriver((Driver)driverClass.newInstance());
            DBConnection = DriverManager.getConnection(
                            DSN, username, password);
        } catch (SQLException sqle) {
            throw new BackEndHandlerInitException( exceptionPrefix + sqle );
        } catch ( ClassNotFoundException cnfe ) {
            throw new BackEndHandlerInitException( exceptionPrefix + cnfe );
        } catch ( IllegalAccessException iae ) {
            throw new BackEndHandlerInitException( exceptionPrefix + iae );
        } catch ( InstantiationException ie ) {
            throw new BackEndHandlerInitException( exceptionPrefix + ie );
        }

    }
    
    /**
     * Execute an SQL SELECT Statement and return the relative Result-Set
     *
     * @param sqlQuery
     * @return ResultSet of the Query
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     */
    private ResultSet getData(String sqlQuery) throws BackEndHandlerGenericException {
        Statement stat = null;
        ResultSet rset = null;

        /* Query Execution */
        try {
            stat = DBConnection.createStatement();
            rset = stat.executeQuery(sqlQuery);
        } catch (SQLException sqle) {
            throw new BackEndHandlerGenericException( exceptionPrefix + sqle );
        }
        return rset;
    }
    
    /**
     * Execute an SQL INSERT/UPDATE/DELETE Statement
     *
     * @param sqlQuery
     * @return Rows affected from the SQL Statement
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     */
    private int execute(String sqlQuery) throws BackEndHandlerGenericException {
        Statement stat = null;
        int result;
        
        /* Query Execution */
        try {
            stat = DBConnection.createStatement();
            result = stat.executeUpdate(sqlQuery);
        } catch (SQLException sqle) {
            throw new BackEndHandlerGenericException( exceptionPrefix + sqle );
        }
        return result;
    }
    
    /**
     * Esegue uno Statement di tipo SELECT sul DB.
     *
     * Permette di specificare solo la Tabella.
     * Seleziona tutte le colonne.
     *
     * @param tableName Nome della Tabella
     * @return ResultSet della Query
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     */
    public ResultSet select(String tableName) 
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        return select( tableName, null, null );
    }
    
    /**
     * Esegue uno Statement di tipo SELECT sul DB.
     *
     * Permette di specificare sia la Tabella,
     * sia le Colonne.
     *
     * @param tableName Nome della Tabella
     * @param columnsName ArrayList contenente le Stringhe dei nomi 
     * delle colonne da Selezionare
     *
     * @return ResultSet della Query
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public ResultSet select(String tableName, ArrayList columnsName) 
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        return select( tableName, columnsName, null );
    }
    
    /**
     * Esegue uno Statement di tipo SELECT sul DB.
     *
     * Permette di specificare sia la Tabella,
     * sia le Colonne, sia semplici condizioni di selezione 
     * ("WHERE"), costruite per uguaglianza. 
     *
     * @param tableName Nome della Tabella
     * @param columnsName ArrayList contenente le Stringhe dei nomi 
     * delle colonne da Selezionare
     *
     * @param whereValues HashMap che descrive un semplice critero di selezione
     * per uguaglianza. 
     * Es. "chiave" => "valore" diventa "WHERE ... chiave = 'valore'".
     *
     * @return ResultSet della Query
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public ResultSet select(String tableName, ArrayList columnsName, HashMap whereValues) 
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        if ( columnsName != null && columnsName.size() < 1 ) {
            throw new BackEndHandlerSQLCreationException( exceptionPrefix + 
                    "SELECT Creation Exception - No \"//Columns/Column/Name\" founded\n");
        } else {
            StringBuffer query = new StringBuffer();
            Iterator colIter;
            StringBuffer whereConditions = new StringBuffer();
            Set colSet = null;
            Iterator colSetIter = null;
            String currColumn;
            
            query.append(" SELECT ");
            
            if ( columnsName == null ) {
                query.append(" * ");
            } else {
                colIter = columnsName.iterator();
                
                query.append( (String)colIter.next() );

                while ( colIter.hasNext() ) {
                    query.append( " , " + (String)colIter.next() );
                }
            }
            query.append(" FROM ");
            query.append(tableName);
            
            /// WHERE CONDITIONS
            if ( whereValues != null && whereValues.size() > 0  ) {
                // Retrieve Columns Name
                colSet = whereValues.keySet();
                colSetIter = colSet.iterator();

                // Construct Updates to be done
                currColumn = (String)colSetIter.next();
                whereConditions.append( currColumn + " = " + 
                        " '"+(String)whereValues.get(currColumn)+"' " );

                while ( colSetIter.hasNext() ) {
                    currColumn = (String)colSetIter.next();
                    whereConditions.append( " AND " + currColumn + " = " + 
                            " '"+(String)whereValues.get(currColumn)+"' " );
                }
                
                query.append( " WHERE " );
                query.append( whereConditions );
            }
            
            return getData( query.toString() );
        }
    }
    
    /**
     * Esegue uno Statement di tipo INSERT sul DB.
     *
     * @param tableName Nome della Tabella
     * @param values HashMap con i valori da Inserire, indicizzati in base 
     * al nome della rispettiva Colonna
     *
     * @return Numero di righe coinvolte nell'Insert (normalmente, sempre 1)
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public int insert(String tableName, HashMap values) 
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        if ( values == null || values.size() < 1 ) {
            throw new BackEndHandlerSQLCreationException( exceptionPrefix + 
                    "INSERT Creation Exception - No \"//Columns/Column/Value\" founded\n");
        } else {
            StringBuffer query = new StringBuffer();
            
            StringBuffer columns = new StringBuffer();
            StringBuffer colValues = new StringBuffer();
            String currColumn = null;
            
            // Retrieve Columns Name
            Set colSet = values.keySet();
            Iterator colSetIter = colSet.iterator();
                        
            // Construct braces with Columns and Values
            currColumn = (String)colSetIter.next();
            columns.append( " ( " + currColumn );
            colValues.append( " ( " + 
                    " '"+(String)values.get(currColumn)+"' " );
            
            while ( colSetIter.hasNext() ) {
                currColumn = (String)colSetIter.next();
                columns.append( ", " + currColumn );
                colValues.append( ", " + 
                        " '"+(String)values.get(currColumn)+"' " );
            }
            
            columns.append( " ) " );
            colValues.append( " ) " );
            
            // Construct the final SQL INSERT Statement
            query.append(" INSERT INTO ");
            query.append( tableName );
            query.append( columns );
            query.append( " VALUES " );
            query.append( colValues );

            return execute( query.toString() );
        }
    }
    
    /**
     * Esegue uno Statement di tipo UPDATE sul DB.
     *
     * Permette di specificare la Tabella e come aggiornare
     * i relativi campi: non essendo specificata una condizione
     * di selezione ("WHERE"), vengono aggiornate tutte le righe.
     *
     * @param tableName Nome della Tabella
     * @param values HashMap con i valori da Aggiornare, indicizzati in base 
     * al nome della rispettiva Colonna
     *
     * @return Numero di righe coinvolte nell'Update 
     * (normalmente, il numero di righe presenti nella Tabella)
     *
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public int update(String tableName, HashMap values)
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        return update(tableName, values, null);
    }
    
    /**
     * Esegue uno Statement di tipo UPDATE sul DB.
     *
     * Permette di specificare la Tabella e come aggiornare
     * i relativi campi. Inoltre, e' possibile definire
     * una semplice condizione di selezione ("WHERE"), costruita
     * per uguaglianza.
     *
     * @param tableName Nome della Tabella
     * @param values HashMap con i valori da Aggiornare, indicizzati in base 
     * al nome della rispettiva Colonna
     *
     * @param whereValues HashMap che descrive un semplice critero di selezione
     * per uguaglianza. 
     * Es. "chiave" => "valore" diventa "WHERE ... chiave = 'valore'".
     *
     * @return Numero di righe coinvolte nell'Update 
     *
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public int update(String tableName, HashMap values, HashMap whereValues)
        throws BackEndHandlerGenericException, BackEndHandlerSQLCreationException {
        
        if ( values == null || values.size() < 1 ) {
            throw new BackEndHandlerSQLCreationException( exceptionPrefix + 
                    "UPDATE Creation Exception - No \"//Columns/Column/Value\" founded\n");
        } else {
            StringBuffer query = new StringBuffer();
            
            StringBuffer updates = new StringBuffer();
            StringBuffer whereConditions = new StringBuffer();
            String currColumn = null;
            Set colSet = null;
            Iterator colSetIter = null;
            
            /// UPDATES
            // Retrieve Columns Name
            colSet = values.keySet();
            colSetIter = colSet.iterator();
                        
            // Construct Updates to be done
            currColumn = (String)colSetIter.next();
            updates.append( currColumn + " = " + 
                    " '"+(String)values.get(currColumn)+"' " );
                        
            while ( colSetIter.hasNext() ) {
                currColumn = (String)colSetIter.next();
                updates.append( ", " + currColumn + " = " + 
                        " '"+(String)values.get(currColumn)+"' " );
            }
            
            /// WHERE CONDITIONS
            if ( whereValues != null && whereValues.size() > 0  ) {
                // Retrieve Columns Name
                colSet = whereValues.keySet();
                colSetIter = colSet.iterator();

                // Construct Updates to be done
                currColumn = (String)colSetIter.next();
                whereConditions.append( currColumn + " = " + 
                        " '"+(String)whereValues.get(currColumn)+"' " );

                while ( colSetIter.hasNext() ) {
                    currColumn = (String)colSetIter.next();
                    whereConditions.append( " AND " + currColumn + " = " + 
                            " '"+(String)whereValues.get(currColumn)+"' " );
                }
            }
            
            // Construct the final SQL INSERT Statement
            query.append(" UPDATE ");
            query.append( tableName );
            query.append(" SET ");
            query.append( updates );
            if ( whereConditions.length() > 3 ) {
                query.append( " WHERE " );
                query.append( whereConditions );
            }

                System.out.println("Query. " + query.toString() );
            return execute( query.toString() );
        }
    }
    
    /**
     * Esegue uno Statement di tipo DELETE sul DB.
     *
     * Permette di specificare la Tabella: cancella tutte le tuple (righe).
     *
     * @param tableName Nome della Tabella
     *
     * @return Numero di righe coinvolte nell'Delete 
     *
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public int delete( String tableName ) 
        throws BackEndHandlerGenericException {
        
        return delete( tableName,  null );
    }
    
    /**
     * Esegue uno Statement di tipo DELETE sul DB.
     *
     * Permette di specificare la Tabella e una semplice condizione 
     * di selezione ("WHERE"), costruita per uguaglianza.
     *
     * @param tableName Nome della Tabella
     * @param whereValues HashMap che descrive un semplice critero di selezione
     * per uguaglianza. 
     * Es. "chiave" => "valore" diventa "WHERE ... chiave = 'valore'".
     *
     * @return Numero di righe coinvolte nell'Delete 
     *
     * @exception BackEndHandlerGenericException Nel caso in cui ci sono dei 
     * problemi durante l'esecuzione della Query
     *
     * @exception BackEndHandlerSQLCreationException Se i Parametri sono errati 
     * e non permettono di costruire una Query ben formata
     */
    public int delete(String tableName, HashMap whereValues)
        throws BackEndHandlerGenericException {
        
        StringBuffer query = new StringBuffer();

        StringBuffer whereConditions = new StringBuffer();
        String currColumn = null;
        Set colSet = null;
        Iterator colSetIter = null;

        /// WHERE CONDITIONS
        if ( whereValues != null && whereValues.size() > 0  ) {
            // Retrieve Columns Name
            colSet = whereValues.keySet();
            colSetIter = colSet.iterator();

            // Construct Updates to be done
            currColumn = (String)colSetIter.next();
            whereConditions.append( currColumn + " = " + 
                    " '"+(String)whereValues.get(currColumn)+"' " );

            while ( colSetIter.hasNext() ) {
                currColumn = (String)colSetIter.next();
                whereConditions.append( " AND " + currColumn + " = " + 
                        " '"+(String)whereValues.get(currColumn)+"' " );
            }
        }

        // Construct the final SQL INSERT Statement
        query.append(" DELETE FROM ");
        query.append( tableName );
        if ( whereConditions.length() > 3 ) {
            query.append( " WHERE " );
            query.append( whereConditions );
        }

        return execute( query.toString() );
    }
    
    /*
    public Object sql(String sqlQueryString) {
        
    }
    */
    
    public String toString() {
        StringBuffer result = new StringBuffer("BackEndHandlerImpl: DataBase Wrapper\n");
        result.append( "- DSN: " + DSN + "\n");
        result.append( "- username: " + username + "\n");
        result.append( "- password: " + ";-)" + "\n");
        result.append( "- JDBCDriverName: " + JDBCDriverName + "\n");
        result.append( "- Loaded Property file path: " + loadedPropertyFilePath + "\n");
        
        return result.toString();
    }
}
