/*
 * ServiceMediatorImpl.java
 *
 * Created on June 12, 2005, 3:58 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server.service;

import org.jaebi.server.core.User;
import org.jaebi.server.service.exception.BackEndHandlerInitException;
import org.jaebi.server.service.exception.CommandCreationException;
import org.jaebi.server.service.exception.CommandExecException;
import org.jaebi.server.service.exception.ServiceMediatorExecException;
import org.jaebi.server.service.exception.ServiceMediatorInitException;
import org.jaebi.server.service.exception.XRequestCommandFactoryInitException;
import org.jaebi.server.service.xml.JAXBMarshallerWrapper;
import org.jaebi.server.service.xml.JAXBMarshallerWrapperImpl;
import org.jaebi.server.service.xml.JAXBXResponseFactory;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperInitException;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperMarshallException;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperUnmarshallException;
import org.jaebi.server.service.xml.exception.JAXBXResponseFactoryBuildingException;
import org.jaebi.server.service.xml.xrequest.Xrequest;
import org.jaebi.server.service.xml.xresponse.Xresponse;

/**
 * @class ServiceMediatorImpl
 * @author detro
 *
 * Implementazione dell'Interfaccia Service Mediator.
 * Abbiamo aggiunto un paio di overloading del metodo "executeService"
 * per comodita'.
 *
 * Questa Interfaccia e una "proposta" al programmatore Client: puo' anche
 * essere ignorata, utilizzando le Classi del Package Service autonomamente.
 */
public class ServiceMediatorImpl implements ServiceMediator {
    private final BackEndHandler backEndHandler;
    private static JAXBMarshallerWrapper xrequestMarshWrap;
    private static JAXBMarshallerWrapper xresponseMarshWrap;
    private static XRequestCommandFactory commFactory;
	private static JAXBXResponseFactory xresponseFactory;
    private static final String exceptionPrefix = 
		"[ServiceMediatorImpl Exception]: ";
	private final CommandValidator validator;
    
    /**
     * Creates a new instance of ServiceMediatorImpl.
     *
     * Istanzia tutte le Classi necessarie per fornire il Service.
     * @param backEndPropertiesFilePath File Path al file di configurazione
     * del BackEndHandler
     * 
     * @exception ServiceMediatorInitException Errore durante l'inizializzazione
     */
	public ServiceMediatorImpl(String backEndPropertiesFilePath)
        throws ServiceMediatorInitException {
        
		validator = new XRequestCommandValidator();
		
		try {
			backEndHandler = new BackEndHandlerImpl(backEndPropertiesFilePath);
			xrequestMarshWrap = new JAXBMarshallerWrapperImpl(
					"org.jaebi.server.service.xml.xrequest");
			xresponseMarshWrap = new JAXBMarshallerWrapperImpl(
					"org.jaebi.server.service.xml.xresponse");
			commFactory = new XRequestCommandFactory(backEndHandler);
			xresponseFactory = new JAXBXResponseFactory();
		} catch ( BackEndHandlerInitException behie ) {
		    throw new ServiceMediatorInitException( exceptionPrefix 
		            + behie );
		} catch ( JAXBMarshallerWrapperInitException jmarinite) {
		    throw new ServiceMediatorInitException( exceptionPrefix 
		            + jmarinite );
		} catch ( XRequestCommandFactoryInitException xreqinite) {
		    throw new ServiceMediatorInitException( exceptionPrefix 
		            + xreqinite );
		}
    }
    
	/**
     * Creates a new instance of ServiceMediatorImpl.
     *
     * Istanzia tutte le Classi necessarie per fornire il Service.
     * 
     * @exception ServiceMediatorInitException Errore durante l'inizializzazione
     */
	public ServiceMediatorImpl()
		throws ServiceMediatorInitException {
		
		this("BackEndHandlerImpl.properties");
	}
	
    public Object executeService(User user, Object request) 
        throws ServiceMediatorExecException {
        
        Xrequest newReq = null;
        XRequestCommand newCommand = null;
        Xresponse newRes = null;
        
        /* Controlla il tipo in INPUT e, se si tratta di String, 
         * esegue l'unmarshalling in un Oggetto di tipo Xrequest */
        if ( request instanceof String ) {
            try {
                newReq = (Xrequest)xrequestMarshWrap.unmarshall(
                        (String)request);
            } catch ( JAXBMarshallerWrapperUnmarshallException unmarshe ) {
                throw new ServiceMediatorExecException( exceptionPrefix 
                        + unmarshe );
            }
        } else {
            newReq = (Xrequest)request;
        }
        
        /* Creazione nuovo XRequestCommand */
        try {
            newCommand = commFactory.createCommand(newReq);
        } catch ( CommandCreationException cce) { 
            throw new ServiceMediatorExecException( exceptionPrefix + cce );
        }
		
		/* Validazione Command */
		if ( validator.validate(user, newCommand) ) {
        		/* Esecuzione dell'XRequestCommand */
			try {
			    newRes = (Xresponse)newCommand.execute();
			} catch ( CommandExecException cee ) {
			    throw new ServiceMediatorExecException( exceptionPrefix + cee );
			}
		} else {
			/* Creazione Xresponse di errore per mancata Validazione */
			String errMessage = "This is not a Valid Request for this User.";
			try {
				newRes = (Xresponse)xresponseFactory.buildXresponse(false, errMessage );
			} catch (JAXBXResponseFactoryBuildingException e) {
				throw new ServiceMediatorExecException( exceptionPrefix + e );
			}
		}
		
		return newRes;
    }
    
    public String executeServiceToString(User user, Object request)
        throws ServiceMediatorExecException {
        
        Xresponse xresponseResult = null;
        String result = null;
        
        try {
            xresponseResult = (Xresponse)executeService(user, request);
            // xresponseMarshWrap.setFormattedMarshalling(true); //TODO Da problemi di parsing al client J2ME
            result = xresponseMarshWrap.marshall(xresponseResult);
        } catch ( ServiceMediatorExecException smee ) {
            throw new ServiceMediatorExecException( exceptionPrefix 
                        + smee );
        } catch ( JAXBMarshallerWrapperMarshallException marshe ) {
                throw new ServiceMediatorExecException( exceptionPrefix 
                        + marshe );
        }
        
        return result;
    }
}
