/*
 * XRequestCommand.java
 *
 * Created on June 5, 2005, 3:06 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.jaebi.server.service.exception.CommandExecException;
import org.jaebi.server.service.exception.XRequestCommandInitException;
import org.jaebi.server.service.xml.JAXBMarshallerWrapper;
import org.jaebi.server.service.xml.JAXBMarshallerWrapperImpl;
import org.jaebi.server.service.xml.JAXBXResponseFactory;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperInitException;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperMarshallException;
import org.jaebi.server.service.xml.xrequest.Column;
import org.jaebi.server.service.xml.xrequest.Xrequest;

/**
 * @class XRequestCommand
 * @author detro
 * 
 * Classe astratta per Command Xrequest.
 * Questa Abstract Class raccoglie le caratteristiche comuni
 * a tutte le classi XRequestCommand, evitando inutili ridondanze di
 * codice.
 *
 * Per creare un XRequestCommand particolare, e' necessario implementare solo
 * 2 metodi:
 * <ul>
 *      <li>Un costruttore che esegua <code>super(xrequestObj, backEndHandler)</code></li>
 *      <li>Un metodo <code>execute</code> in cui viene eseguito il command</li>
 * </ul>
 * Le classi cosi' create hanno a disposizione tutto quello di cui
 * necessitano, ereditandolo come campi "protected".
 * 
 * Inoltre, la Classe prevede un sistema di Caching che evita di ricalcolare
 * i risultati dell'analisi dell'XML ad ogni richiesta. Questo, nel caso di un
 * uso massiccio dei Command, migliora sostanzialmente le prestazioni.
 * Questo approccio e' favorito dal fatto che tutti gli XRequestCommand
 * si basano sullo stesso XML.
 * 
 * In alcuni casi, si e' usata una notazione "XPath" per indicare i Nodi.
 */
 abstract public class XRequestCommand extends Command {
    /** JAXBXResponseFactory utile a tutti gli XRequestCommand
     * per produrre oggetti XResponse */
    protected static final JAXBXResponseFactory xresponseFactory = 
            new JAXBXResponseFactory();
     
    /** BackEndHandler su cui viene eseguito Command */
    protected final BackEndHandlerImpl backEndHandler;
    
    /** Oggetto Xrequest da Wrappare, che descrive
     * il Command che lo User ha richiesto di eseguire */
    protected final Xrequest xrequestObj;
    
    /** Variabile "cache" che memorizza i valori contenuti in
     * "<code>//Columns/Column/Name</code>" */
    private ArrayList columnsName = null;
    
    /** Variabile "cache" che memorizza i valori contenuti in
     * "<code>//Columns/Column/Value</code>" utilizzando come chiave
     * il valore del relativo "<code>//Columns/Column/Value/../Name</code>" */
    private HashMap columnsValue = null;
    
    /** Variabile "cache" che memorizza i valori contenuti in
     * "<code>//Where/Column/Name</code>" */
    private ArrayList whereColumnsName = null;
    
    /** Variabile "cache" che memorizza i valori contenuti in
     * "<code>//Where/Column/Value</code>" utilizzando come chiave
     * il valore del relativo "<code>//Where/Column/Value/../Name</code>" */
    private HashMap whereColumnsValue = null;
    
    protected static String exceptionPrefix = "[XRequestCommand Exception]: ";
    
    /**
     * Costruttore
     *
     * @param newXrequestObj Oggetto di tipo Xrequest che sara' "wrappato" dal Command
     */
    public XRequestCommand(Xrequest newXrequestObj, BackEndHandlerImpl newBackEndHandlerImpl) 
        throws XRequestCommandInitException {
        
        if ( newXrequestObj == null || newBackEndHandlerImpl == null ) {
            throw new XRequestCommandInitException(exceptionPrefix + 
                    "Init parameter error\n");
        }
        xrequestObj = newXrequestObj;
        backEndHandler = newBackEndHandlerImpl;
        
    }
    
    // Ereditato da Command
    public abstract Object execute() throws CommandExecException; 
    
    /**
     * Ritorna il valore dell'attributo "<code>/xrequest/@type</code>"
     */
    public String getType() {
        return xrequestObj.getType();
    }
    
    /**
     * Table si cui viene eseguita l'XRequestCommand
     * 
     * @return Nome Tabella coinvolta dal Command
     */
    public String getAffectedTable() {
        return xrequestObj.getTable();
    }
    
    /**
     * Costruisce un ArrayList con i valori dei TAG "Name" dentro
     * i TAG "Column" contenuti in "Columns" o "Where".
     *
     * Il metodo e' privato e generico, cosi' da poter semplificare
     * e snellire la stesura del codice.
     *
     * @param columns Lista di Column da analizzare
     * @return ArrayList di Stringhe di "Name"
     */
    private ArrayList getNameFromColumnList(List columns) {
        ArrayList result = new ArrayList();
        if ( columns != null ) {
            Iterator iter = columns.iterator();
            Column column = null;

            while ( iter.hasNext() ) {
                column = (Column)iter.next();
                result.add( column.getName() );
            }
        }

        return result;
    }
    
    /**
     * Costruisce un HashMap con i valori dei TAG "Value", 
     * con chiavi i valori dei rispettivi TAG "Name", dentro
     * i TAG "Column" contenuti in "Columns" o "Where"
     *
     * Il metodo e' privato e generico, cosi' da poter semplificare
     * e snellire la stesura del codice.
     *
     * @param columns List di Column da Analizzare
     * @return HashMap di Stringhe di "Value" con chiave "Name"
     */
    private HashMap getValueFromColumnList(List columns) {
        HashMap result = new HashMap();
        if ( columns != null ) {
            Iterator iter = columns.iterator();
            Column column = null;

            while ( iter.hasNext() ) {
                column = (Column)iter.next();
                result.put(column.getName(), column.getValue());
            }
        }
        
        return result;
    }
    
    /**
     * Recupero valori di "<code>//Columns/Column/Name</code>"
     *
     * @return ArrayList dei valori contenuti nei nodi "Name", contenuti in "Column"
     */
    public ArrayList getColumnsName() {
        if ( columnsName == null ) {
            if ( xrequestObj.getColumns() != null ) {
                columnsName = getNameFromColumnList( 
                        xrequestObj.getColumns().getColumn() ); // Caching...
            } else
                columnsName = new ArrayList();
        }
        return columnsName; // Using Cache...
    }
    
    /**
     * Recupero valori di "<code>//Columns/Column/Value</code>".
     *
     * Indicizzati in base ai rispettivi 
     * "<code>//Columns/Column/Value/../Name</code>"
     *
     * @return HashMap dei valori contenuti nei nodi "Value", 
     * contenuti in "Column"
     */
    public HashMap getColumnsValue() {
        if ( columnsValue == null ) {
            if ( xrequestObj.getColumns() != null ) {
                columnsValue = getValueFromColumnList( 
                        xrequestObj.getColumns().getColumn() ); // Caching...
            } else
                columnsValue = new HashMap();
        }
        return columnsValue; // Using Cache...
    }
    
    /**
     * Recupero valori di "<code>//Where/Column/Name</code>"
     *
     * @return ArrayList dei valori contenuti nei nodi "Name", contenuti in "Column"
     */
    public ArrayList getWhereColumnsName() {
        if ( whereColumnsName == null ) {
            if ( xrequestObj.getWhere() != null ) {
                whereColumnsName = getNameFromColumnList( 
                        xrequestObj.getWhere().getColumn() ); // Caching...
            } else
                whereColumnsName = new ArrayList();
        }
        return whereColumnsName; // Using Cache...
    }
    
    /**
     * Recupero valori di "<code>//Where/Column/Value</code>".
     *
     * Indicizzati in base ai rispettivi 
     * "<code>//Where/Column/Value/../Name</code>"
     *
     * @return HashMap dei valori contenuti nei nodi "Value", contenuti in "Column"
     */
    public HashMap getWhereColumnsValue() {
        if ( whereColumnsValue == null ) {
            if ( xrequestObj.getWhere() != null ) {
                whereColumnsValue = getValueFromColumnList( 
                        xrequestObj.getWhere().getColumn() ); // Caching...
            } else
                whereColumnsValue = new HashMap();
        }
        return whereColumnsValue; // Using Cache...
    }
    
    public String toString() {
        JAXBMarshallerWrapper marshWrap = null;
        String result = null;
        
        try {
            marshWrap = new JAXBMarshallerWrapperImpl("org.jaebi.server.service.xml.xrequest");
            marshWrap.setFormattedMarshalling(true);
            result = marshWrap.marshall( xrequestObj );
        } catch ( JAXBMarshallerWrapperMarshallException e ) {
            return "<XRequestCommand> toString conversion (marshalling) exception:\n" + e;
        } catch ( JAXBMarshallerWrapperInitException e ) {
            return "<XRequestCommand> toString conversion (Marshall Wrapper Init) exception:\n" + e;
        }
        
        return result;
    }
}
