/*
 * JAXBMarshallerWrapperImpl.java
 *
 * Created on May 29, 2005, 7:02 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server.service.xml;

import java.io.File;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperInitException;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperMarshallException;
import org.jaebi.server.service.xml.exception.JAXBMarshallerWrapperUnmarshallException;

/**
 * @class JAXBMarshallerWrapperImpl
 * @author detro
 * 
 * JAXBMarshallerWrapperImpl wrappa il Marshaller e l'Unmarshaller per
 * l'XML selezionato.
 * Questa tecnica semplifica molto l'utilizzo della libreria JAXB, incapsulando
 * tutte le operazioni necessarie per l'istanziazione di Marshaller e
 * Unmarshaller.
 */
public class JAXBMarshallerWrapperImpl implements JAXBMarshallerWrapper {
    /** JAXBContext */
    private final JAXBContext context;
    /** Marshaller */
    private final Marshaller marshaller;
    /** Marshaller */
    private final Unmarshaller unmarshaller;
    /** Formatted Mashalling Setting */
    private boolean formattedMarshalling;
    /** Prefisso Messaggio da inserire nelle Eccezioni lanciate */
    private static String exceptionPrefix = "[JAXBMarshallerWrapperImpl Exception]: ";
        
    /**
     * Creates a new instance of JAXBMarshallerWrapperImpl.
     * 
     * @param packageName Stringa del nome del package che contiene il file 
     * jaxb.properties relativo al particolare XSD
     * 
     * @exception JAXBMarshallerWrapperUnmarshallException Se ci sono problemi durante 
     * l'istanziazione dei Marshaller
     */
    public JAXBMarshallerWrapperImpl(String packageName) 
        throws JAXBMarshallerWrapperInitException {
        
        try {
            // JAXBContext
            context = JAXBContext.newInstance(packageName);
            // Marshaller
            marshaller = context.createMarshaller();
            // Marshaller
            unmarshaller = context.createUnmarshaller();
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperInitException(exceptionPrefix + e);
        } 
    }
    
    public String marshall(Object marshallable) 
        throws JAXBMarshallerWrapperMarshallException {
        
        StringWriter writer = new StringWriter();
        
        try {
            marshaller.marshal(marshallable, writer);
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperMarshallException(exceptionPrefix + e);
        }
        
        return writer.toString();
    }
   
    public Object unmarshall(URL url) throws JAXBMarshallerWrapperUnmarshallException {
        Object xmlElement = null; 
        
        try {
            xmlElement = unmarshaller.unmarshal(url);
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperUnmarshallException(exceptionPrefix + e);
        }
        
        return xmlElement;
    }
    
    public Object unmarshall(File file) throws JAXBMarshallerWrapperUnmarshallException {
        Object xmlElement = null; 
        
        try {
            xmlElement = unmarshaller.unmarshal(file);
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperUnmarshallException(exceptionPrefix + e);
        }
        
        return xmlElement;
    }
    
    public Object unmarshall(InputStream inputStream) 
        throws JAXBMarshallerWrapperUnmarshallException {
        
        Object xmlElement = null; 
        
        try {
            xmlElement = unmarshaller.unmarshal(inputStream);
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperUnmarshallException(exceptionPrefix + e);
        }
        
        return xmlElement;
    }
    
    public Object unmarshall(Source source) 
        throws JAXBMarshallerWrapperUnmarshallException {
        
        Object xmlElement = null; 
        
        try {
            xmlElement = unmarshaller.unmarshal(source);
        } catch (JAXBException e) {
            throw new JAXBMarshallerWrapperUnmarshallException(exceptionPrefix + e);
        }
        
        return xmlElement;
    }
    
    public Object unmarshall(String xmlString) 
        throws JAXBMarshallerWrapperUnmarshallException {
        
        /* Creazione StringBuffer che useremo nell'unmarshalling.
         * L'interfaccia della classe Unmashaller non prevede un metodo
         * che prenda in input, direttamente, un oggetto String */
        StringBuffer xmlStr = new StringBuffer( xmlString );
        
        return unmarshall( new StreamSource( new StringReader( xmlStr.toString() ) ) );
    }
    
    public void setFormattedMarshalling(boolean value) 
        throws JAXBMarshallerWrapperMarshallException {
        
        formattedMarshalling = value;
        try {
            if ( formattedMarshalling ) {
                /* XML Formatting Property Setting */
                marshaller.setProperty(
                        Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean("true"));
            } else {
                /* XML Formatting Property Setting */
                marshaller.setProperty(
                        Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean("false"));
            }
        } catch ( JAXBException e ) {
            throw new JAXBMarshallerWrapperMarshallException(exceptionPrefix + e);
        }
    }
    
    public boolean isFormattedMarshalling() {
        return formattedMarshalling;
    }
}
