/*
 * JAXBXResponseFactory.java
 *
 * Created on May 31, 2005, 2:12 PM
 * Created by Detro - 566/2145
 */

package org.jaebi.server.service.xml;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Hashtable;

import javax.xml.bind.JAXBException;

import org.jaebi.server.service.xml.exception.JAXBXResponseFactoryBuildingException;
import org.jaebi.server.service.xml.xresponse.Column;
import org.jaebi.server.service.xml.xresponse.Message;
import org.jaebi.server.service.xml.xresponse.Row;
import org.jaebi.server.service.xml.xresponse.Rowset;
import org.jaebi.server.service.xml.xresponse.Xresponse;

/**
 * @class JAXBXResponseFactory
 * @author detro
 * 
 * Factory per la costruzione semplificata degli oggetti JAXB Xresponse.
 * Ovviamente la sua implementazione dipende strettamente dall'XSD
 * definito per i messaggi di Response.
 */
public class JAXBXResponseFactory {
    /** Object Factory per l'XSD "xresponse" */
    protected static org.jaebi.server.service.xml.xresponse.ObjectFactory objectFactory = null;
    /** Prefisso Messaggio da inserire nelle Eccezioni lanciate */
    private static String exceptionPrefix = "[XResponseFactory Exception]: ";
    /** Hashmap che associa vari tipi SQL ai tipi validi
     * per l'XSD "xresponse" */
    private static Hashtable XResponseTypesMapping = null;
    
    /**
     * Creates a new instance of JAXBXResponseFactory 
     */
    public JAXBXResponseFactory() {
        // Istanziazione Object Factory
        objectFactory = new org.jaebi.server.service.xml.xresponse.ObjectFactory();
        /* Filling della Hashmap di associazione SQL-xresponse column's type:
         * il parametro numerico in input permette di stabilire la capacita' iniziale
         * dell'Hashmap, che e' il numero di Bucket inizialmente istanziati 
         */
        XResponseTypesMapping = new Hashtable(20);
        XResponseTypesMapping.put("bit varying", "bits");
        XResponseTypesMapping.put("bit", "bits");
        XResponseTypesMapping.put("boolean","boolean");
        XResponseTypesMapping.put("character varying", "chars");
        XResponseTypesMapping.put("varchar", "chars");
        XResponseTypesMapping.put("character", "chars");
        XResponseTypesMapping.put("char", "chars");
        XResponseTypesMapping.put("text", "chars");
        XResponseTypesMapping.put("integer", "int");
        XResponseTypesMapping.put("int4", "int");
        XResponseTypesMapping.put("int", "int");
        XResponseTypesMapping.put("real", "float");
        XResponseTypesMapping.put("float", "float");
        XResponseTypesMapping.put("float4", "float");
        XResponseTypesMapping.put("timestamp", "chars");
        XResponseTypesMapping.put("timestamp without time zone", "chars");
        XResponseTypesMapping.put("timestamp with time zone", "chars");
        XResponseTypesMapping.put("timestamptz", "chars");
        XResponseTypesMapping.put("date","chars");
    }
    
    /**
     * Costruisce Oggetto Xresponse che contiene solo
     * un messaggio.
     *
     * E' indicato per operazioni che richiedono solo "conferma" di avvenuta
     * esecuzione o segnalazione d'errore.
     * 
     * @param ok "true" se l'operazione e' riuscita, "false" altrimenti
     * @param message Eventuale messaggio da restituire
     * @return Object da upcastare a Xresponse
     * @exception JAXBXResponseFactoryBuildingException Errore durante 
     * la costruzione dell'oggetto Xresponse.
     */
    public Object buildXresponse(boolean ok, String message) 
        throws JAXBXResponseFactoryBuildingException {
        
        Xresponse xresponse = null;
        Message xrespMessage = null;
        
        try {
            // Creazione nuovo oggetto Xresponse
            xresponse = objectFactory.createXresponse();
            
            // Creazione nuovo oggetto Message
            xrespMessage = objectFactory.createMessage();
            // Settaggio attributo "result" per l'oggetto Message
            if ( ok ) xrespMessage.setResult("ok");
            else xrespMessage.setResult("error");
            // Settaggio contenuto per oggetto Message
            xrespMessage.getContent().add(message);
            
            // Aggiunta Oggetto Message nell'oggetto Xresponse
            xresponse.setMessage(xrespMessage);
        } catch ( JAXBException e ) {
            throw new JAXBXResponseFactoryBuildingException(exceptionPrefix + e);
        }
        
        return (Object)xresponse;
        
    }
    
    /**
     * Costruisce Oggetto Xresponse che contiene il risultato di una SELECT.
     *
     * Permette di Mappare nell'XSD "xresponse" il risultato di una operazione
     * di selezione su dei dati, trasformando un ResultSet.
     * 
     * @param resultSet Risultato di una operazione di Selezione du dei dati
     * @return Object da upcastare a Xresponse
     * @exception JAXBXResponseFactoryBuildingException Errore durante 
     * la costruzione dell'oggetto Xresponse.
     */
    public Object buildXresponse(ResultSet resultSet) 
        throws JAXBXResponseFactoryBuildingException {
        
        Xresponse xresponse = null;
        Rowset rowset = null;
        Row row = null;
        Column column = null;
        String columnTypeStr;
        String columnNameStr;
        String columnValueStr;
        
        ResultSetMetaData resSetMeta = null;
        int columnsNumber = 0;
        
        try {
            // Creazione nuovo oggetto Xresponse
            xresponse = objectFactory.createXresponse();
            
            // Creazione nuovo oggetto Rowset
            rowset = objectFactory.createRowset();
            
            // Recupero MetaData dal ResultSet
            resSetMeta = resultSet.getMetaData();
            // Recuper numero delle Colonne del ResultSet
            columnsNumber = resSetMeta.getColumnCount();
            
            while ( resultSet.next() ) {
                // Creazione nuovo oggetto Row
                row = objectFactory.createRow();
                                
                for ( int i = 0; i < columnsNumber; i++ ) {
                    // Creazione nuovo oggetto Column
                    column = objectFactory.createColumn();
                    
                    // Recupero nome del Type della Colonna
                    columnTypeStr = resSetMeta.getColumnTypeName(i + 1);
                    /* Settaggio attributo "type" per l'oggetto Column
                     * per mappare i tipi possibili dell'SQL con i tipi
                     * accettati dall'XSD xresponse si usa una Hashmap
                     */                    
                    column.setType( 
                            (String)XResponseTypesMapping.get( 
                                (String) columnTypeStr.toLowerCase()
                                ) 
                            );
                    
                    // Recupero nome della Colonna
                    columnNameStr = resSetMeta.getColumnName(i + 1);
                    // Settaggio valore Oggetto Name contenuto in Column
                    column.setName(columnNameStr);
                    
                    // Recupero valore della Colonna
                    columnValueStr = resultSet.getString(i + 1);
					if ( columnValueStr == null )
						columnValueStr = "NULL";
                    // Settaggio valore Oggetto Value contenuto in Column
                    column.setValue(columnValueStr);
                    
                    // Inserimento Colonna nella Riga
                    row.getColumn().add(column);
                }
                
                // Aggiunto oggetto Row al Rowset
                rowset.getRow().add(row);
            }
            
            
            // Aggiunga Oggetto Message nell'oggetto Xresponse
            xresponse.setRowset(rowset);
        } catch ( JAXBException e ) {
            throw new JAXBXResponseFactoryBuildingException(exceptionPrefix + e);
        } catch (SQLException sqle) {
            throw new JAXBXResponseFactoryBuildingException(exceptionPrefix + sqle);
	}
        
        return (Object)xresponse;
    }
    
}
